//http通訊
#include <HttpClient.h>
#include <WiFi.h>
#include <WiFiClient.h>
//RFID
#include <SPI.h>
#include <MFRC522.h>
//LCD
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

//http
char ssid[] = "LAPTOP-O633K9EF 4268";        // your network SSID (name)
char pass[] = "6W#9305b";           // your network password (use for WPA, or use as key for WEP)
int keyIndex = 0;                   // your network key Index number (needed only for WEP)
int status = WL_IDLE_STATUS;        // Indicator of Wifi status
uint16_t serverPort = 80;           // HTTP 預設連接埠
const char kHostname[] = "172.31.133.17";
const char kPath[] = "/ardiuno/insert_data.php";
const int kNetworkTimeout = 30 * 1000;
const int kNetworkDelay = 1000;

//RFID
#define RST_PIN 6
#define SS_PIN 9
MFRC522 mfrc522(SS_PIN, RST_PIN);

//LCD
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

void setup() {
    Serial.begin(115200);
    SPI.begin(); 
    mfrc522.PCD_Init();
    lcd.begin(16, 2);      
    lcd.backlight();
    lcd.setCursor(0, 0);
    lcd.print("Scan a Tag!");
    connectToWiFi(); // 連接到 WiFi
}

void loop() {
    if (!mfrc522.PICC_IsNewCardPresent()) {
        return; // 沒有檢測到新卡片，返回
    }
    if (!mfrc522.PICC_ReadCardSerial()) {
        return; // 無法讀取卡片，返回
    }

    lcd.clear(); // 清除 LCD 顯示
    lcd.setCursor(0, 0);
    lcd.print("UID:");

    // 將 UID 轉換為字串
    String uid = "";
    for (byte i = 0; i < mfrc522.uid.size; i++) {
        if (mfrc522.uid.uidByte[i] < 0x10) {
          lcd.print("0");
          uid += "0"; 
        }
        lcd.print(mfrc522.uid.uidByte[i], HEX); 
        uid += String(mfrc522.uid.uidByte[i], HEX); 
        if (i != mfrc522.uid.size - 1) {
          uid += ":"; 
          lcd.print(":");
        }
    }

    // 將 UID 傳送到伺服器
    if (WiFi.status() == WL_CONNECTED) {
        sendPostRequest(uid); // 發送 POST 請求
    } else {
        Serial.println("WiFi Disconnected. Reconnecting...");
        connectToWiFi();
    }
    delay(10000); 
}

void connectToWiFi() {
  Serial.print("Connecting to WiFi...");
  while (WiFi.begin(ssid, pass) != WL_CONNECTED) {
    Serial.print(".");
    delay(5000); 
  }
  Serial.println("\nConnected!");
  printWifiStatus();
}

void sendPostRequest(String uid) {
  WiFiClient client;
  HttpClient http(client);

  String postData = "uid=" + uid;

  Serial.println("Sending POST request...");
  http.beginRequest();
  http.post(kHostname, serverPort, kPath);
  http.sendHeader("Content-Type", "application/x-www-form-urlencoded");
  http.sendHeader("Content-Length", postData.length());
  http.endRequest();
  http.print(postData); // 傳送資料

  // 處理伺服器回應
  int statusCode = http.responseStatusCode();
  Serial.print("Response Status Code: ");
  Serial.println(statusCode);

  Serial.println("Response Body:");
  while (http.available()) {
      Serial.print((char)http.read());
  }

  http.stop(); // 關閉 HTTP 連線
}

void printWifiStatus() {
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());
  Serial.print("Signal Strength (RSSI): ");
  Serial.print(WiFi.RSSI());
  Serial.println(" dBm");
}